@@ -15,11 +15,12 @@ from typing import TypeVar
 
 from d2b import defaults
 from d2b.plugins import pm
+from d2b.utils import associated_nii_ext
 from d2b.utils import prepend
 from d2b.utils import rsync
 from d2b.utils import splitext
 
-__version__ = "1.0.0"
+__version__ = "1.1.2"
 
 
 T = TypeVar("T")
@@ -300,27 +301,19 @@ class Matcher:
             if intended_for is None:
                 continue
 
-            elif isinstance(intended_for, int):
-                target_acq = self._resolve_intended_for(self.acquisitions, intended_for)
-                if target_acq is None:
+            elif isinstance(intended_for, int) or isinstance(intended_for, str):
+                fp = self._resolve_intended_for_path(acq, intended_for)
+                if fp is None:
                     continue
-                # TODO: the suffix could in theory be wrong here (i.e. just .nii)
-                bids_path = target_acq.dst_root.with_suffix(".nii.gz")
-                fp = bids_path.relative_to(target_acq.participant.subject_directory)
                 acq.data["IntendedFor"] = str(fp)
 
             elif isinstance(intended_for, list):
-                for desc_idx in intended_for:
-                    target_acq = self._resolve_intended_for(
-                        self.acquisitions,
-                        desc_idx,
-                    )
-                    if target_acq is None:
+                for _intended_for in intended_for:
+                    fp = self._resolve_intended_for_path(acq, _intended_for)
+                    if fp is None:
                         continue
                     if acq.data.get("IntendedFor") is None:
                         acq.data["IntendedFor"] = []
-                    bids_path = target_acq.dst_root.with_suffix(".nii.gz")
-                    fp = bids_path.relative_to(target_acq.participant.subject_directory)
                     acq.data["IntendedFor"].append(str(fp))
 
             else:
@@ -329,12 +322,41 @@ class Matcher:
                     f"description at index [{description.index}]",
                 )
 
+    def _resolve_intended_for_path(
+        self,
+        acq: Acquisition,
+        intended_for: int | str,
+    ) -> Path | None:
+        target_acq = self._resolve_intended_for(self.acquisitions, intended_for)
+        if target_acq is None:
+            return
+        ext = associated_nii_ext(target_acq.src_file)
+        if ext is None:
+            self._log_associated_nii_not_found(acq, target_acq)
+            return
+        bids_path = target_acq.dst_root.with_suffix(ext)
+        return bids_path.relative_to(target_acq.participant.subject_directory)
+
     @staticmethod
     def _resolve_intended_for(
         acquisitions: list[Acquisition],
-        intended_for: int,
+        intended_for: int | str,
     ) -> Acquisition | None:
-        match_refs = [a for a in acquisitions if a.description.index == intended_for]
+        if isinstance(intended_for, int):
+            match_refs = [
+                a for a in acquisitions if a.description.index == intended_for
+            ]
+        elif isinstance(intended_for, str):
+            match_refs = [
+                a for a in acquisitions if a.description.data.get("id") == intended_for
+            ]
+        else:
+            msg = (
+                "IntendedFor field must be of type int | str | (int | str)[]. "
+                f"Found [{intended_for!r}]"
+            )
+            raise ValueError(msg)
+
         if len(match_refs) == 0:
             # this case can happen if IntendedFor references a
             # description for which there was no match. In this
@@ -344,6 +366,15 @@ class Matcher:
 
         return match_refs[0]
 
+    def _log_associated_nii_not_found(self, acq: Acquisition, target_acq: Acquisition):
+        msg = (
+            f"No NIfTI file associated with file [{target_acq.src_file}]. "
+            "This acquisition was determined to be one of the IntendedFor "
+            f"acquisitions for the file [{acq.src_file}] matching description "
+            f"at index [{acq.description.index}]"
+        )
+        self.logger.warning(msg)
+
 
 class Acquisition:
     """By definition (we declare here) an `Acquisition` is a set of files
@@ -460,7 +491,7 @@ class Description:
         modality_label: str,
         custom_labels: str | dict[str, str] = "",
         sidecar_changes: dict[str, Any] | None = None,
-        intended_for: int | list[int] | None = None,
+        intended_for: int | str | list[int | str] | None = None,
         data: dict[str, Any] = None,
     ):
         self.index = index
@@ -523,14 +554,14 @@ class Description:
 
     @property
     def custom_labels(self):
-        return self._custom_labels
+        return prepend(str(self._custom_labels), "_")
 
     @custom_labels.setter
     def custom_labels(self, val: str | dict[str, str]):
-        _v = val
-        if isinstance(_v, dict):
-            _v = "_".join(f"{k}-{v}" for k, v in _v.items())
-        self._custom_labels = prepend(_v, "_")
+        if isinstance(val, str):
+            self._custom_labels = FilenameEntities.from_string(val)
+        else:
+            self._custom_labels = FilenameEntities(val)
 
     # computed properties
     @property
@@ -556,3 +587,77 @@ def _setup_logging(log_level, logFile=None):
         handler.setFormatter(formatter)
         handler.setLevel("DEBUG")
         logger.addHandler(handler)
+
+
+class FilenameEntities:
+    # the order matters (BIDS v1.6 - https://bids-specification.readthedocs.io/en/v1.6.0/99-appendices/09-entities.html#trc) # noqa: E501
+    KNOWN_ENTITIES = (
+        "sub",
+        "ses",
+        "task",
+        "acq",
+        "ce",
+        "trc",
+        "rec",
+        "dir",
+        "run",
+        "mod",
+        "echo",
+        "flip",
+        "inv",
+        "mt",
+        "part",
+        "recording",
+        "proc",
+        "space",
+        "split",
+        "res",
+        "den",
+        "label",
+        "desc",
+    )
+
+    def __init__(self, entities: dict[str, str]):
+        self.entities = self.parse(entities)
+
+    def __repr__(self):
+        return f"{self.__class__.__name__}({dict(self)!r})"
+
+    def __str__(self):
+        return "_".join(f"{k}-{v}" for k, v in self)
+
+    def __iter__(self) -> Iterator[tuple[str, str]]:
+        _d = self.entities.copy()
+
+        for k in self.KNOWN_ENTITIES:
+            v = _d.pop(k, None)
+            if v is None:
+                continue
+            yield (k, v)
+
+        for k, v in _d.items():
+            yield (k, v)
+
+    @classmethod
+    def from_string(cls, s: str):
+        return cls(cls.parse(s))
+
+    @classmethod
+    def parse(cls, entities: str | dict[str, str]) -> dict[str, str]:
+        if isinstance(entities, dict):
+            return entities
+        entity_strs = entities.strip("_").split("_")
+        return dict(map(cls._split_entity_str, entity_strs))
+
+    @staticmethod
+    def _split_entity_str(s: str) -> tuple[str, str]:
+        parts = s.split("-")
+        if len(parts) < 2:
+            m = (
+                f"Failed parsing entity string [{s}]. "
+                "String must have at least one hyphen."
+            )
+            raise ValueError(m)
+        k = parts[0]
+        v = "".join(filter(str.isalnum, "".join(parts[1:])))
+        return k, v
